@@ -1,21 +1,24 @@
-/* 
+/*
  * i-scream central monitoring system
  * http://www.i-scream.org
- * Copyright (C) 2000-2003 i-scream
+ * Copyright (C) 2000-2004 i-scream
  * 
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+ * 02111-1307 USA
+ *
+ * $Id: disk_stats.c,v 1.49 2004/01/19 16:49:21 tdb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -576,9 +579,13 @@ diskio_stat_t *get_diskio_stats(int *entries){
 		n++;
 	}
 
+	fclose(f);
+	f = NULL;
+
 	if (!has_pp_stats) {
-		/* This is an older kernel without stats in /proc/partitions.
-		   Read what we can from /proc/stat instead. */
+		/* This is an older kernel where /proc/partitions doesn't
+		   contain stats. Read what we can from /proc/stat instead, and
+		   fill in the appropriate bits of the list allocated above. */
 
 		f = fopen("/proc/stat", "r");
 		if (f == NULL) goto out;
@@ -690,6 +697,9 @@ diskio_stat_t *get_diskio_stats_diff(int *entries){
 	}
 
 	diskio_stats_ptr=get_diskio_stats(&disks);
+	if (diskio_stats_ptr == NULL) {
+		return NULL;
+	}
 	diskio_stats_diff_ptr=diskio_stats_diff;
 
 	for(x=0;x<sizeof_diskio_stats_diff;x++){
