@@ -1,7 +1,7 @@
 /*
  * i-scream central monitoring system
  * http://www.i-scream.org
- * Copyright (C) 2000-2002 i-scream
+ * Copyright (C) 2000-2004 i-scream
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,6 +16,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * $Id: saidar.c,v 1.25 2004/01/21 10:27:46 tdb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -446,7 +448,7 @@ int main(int argc, char **argv){
                 switch (c){
                         case 'd':
                                 delay = atoi(optarg);
-				if (delay == 0){
+				if (delay < 1){
 					fprintf(stderr, "Time must be 1 second or greater\n");
 					exit(1);
 				}
